from enum import Enum


class Genre(Enum):
    ACTION = "Action"
    ADULT = "Adult"
    ADVENTURE = "Adventure"
    COMEDY = "Comedy"
    DRAMA = "Drama"
    ECCHI = "Ecchi"
    FANTASY = "Fantasy"
    HAREM = "Harem"
    HORROR = "Horror"
    LOLICON = "Lolicon"
    MARTIAL_ARTS = "Martial Arts"
    MECHA = "Mecha"
    MYSTERY = "Mystery"
    PSYCHOLOGICAL = "Psychological"
    ROMANCE = "Romance"
    SCI_FI = "Sci-Fi"
    SCHOOL_LIFE = "School Life"
    SLICE_OF_LIFE = "Slice of Life"
    SHOTACON = "Shotacon"
    SUPERNATURAL = "Supernatural"
    THRILLER = "Thriller"

    # Demographic Categories
    SHOUJO = "Shoujo"
    SHOUJO_AI = "Shoujo Ai"
    SHOUNEN = "Shounen"
    SHOUNEN_AI = "Shounen Ai"
    SEINEN = "Seinen"
    JOSEI = "Josei"
    MATURE = "Mature"

    # Popular Subgenres
    ISEKAI = "Isekai"
    REINCARNATION = "Reincarnation"
    TIME_TRAVEL = "Time Travel"
    VIDEO_GAME = "Video Game"
    VIRTUAL_REALITY = "Virtual Reality"
    ZOMBIE = "Zombie"
    APOCALYPSE = "Apocalypse"
    POST_APOCALYPSE = "Post-Apocalypse"
    MONSTER = "Monster"
    VAMPIRE = "Vampire"
    GHOST = "Ghost"
    YOKAI = "Yokai"
    MAGIC = "Magic"
    WUXIA = "Wuxia"
    XIANXIA = "Xianxia"
    MILITARY = "Military"
    POLICE = "Police"
    MEDICAL = "Medical"
    MUSIC = "Music"
    SPORTS = "Sports"
    COOKING = "Cooking"
    GENDER_BENDER = "Gender Bender"
    CROSSDRESSING = "Crossdressing"
    REVERSE_HAREM = "Reverse Harem"
    YAOI = "Yaoi"
    YURI = "Yuri"

    # Thematic Categories
    SCHOOL = "School"
    COLLEGE = "College"
    OFFICE_WORKER = "Office Worker"
    DELINQUENT = "Delinquent"
    MAFIA = "Mafia"
    HISTORICAL = "Historical"
    STEAMPUNK = "Steampunk"
    CYBERPUNK = "Cyberpunk"
    BIOPUNK = "Biopunk"
    DIESELPUNK = "Dieselpunk"
    URBAN_FANTASY = "Urban Fantasy"
    DARK_FANTASY = "Dark Fantasy"
    HIGH_FANTASY = "High Fantasy"
    LOW_FANTASY = "Low Fantasy"
    NOIR = "Noir"

    # Mood/Tone
    GORE = "Gore"
    TRAGEDY = "Tragedy"
    HEARTWARMING = "Heartwarming"
    WHOLESOME = "Wholesome"
    SATIRE = "Satire"
    PARODY = "Parody"
    META = "Meta"

    # Format Styles
    FOUR_KOMA = "4-Koma"
    ANTHOLOGY = "Anthology"
    ONESHOT = "Oneshot"
    FULL_COLOR = "Full Color"

    # Special Categories
    ADAPTATION = "Adaptation"
    FANWORK = "Fanwork"
    DOUJINSHI = "Doujinshi"
    OFFICIAL = "Official"
    UPDATING = "Updating"
    WEBTOON = "Webtoon"
    MANHWA = "Manhwa"
    MANHUA = "Manhua"

    UNKNOWN = "Unknown"



from dataclasses import dataclass
from mugennocore.model.interfaces import ISearchObj


@dataclass
class SearchObj:
    """Represents a Search return Object"""

    title: str
    url: str
    cover_url: str
    score: float
    last_chapter: str

    def __str__(self) -> str:
        return f"""
{self.title}
{self.cover_url}
★ - {self.score} | 🕮 - {self.last_chapter}
"""

    def __repr__(self) -> str:
        return f"""
SearchObj(
    title={self.title!r}
    url={self.url!r}
    cover_url={self.cover_url!r}
    score={self.score!r}
    last_chapter={self.last_chapter!r}
)
"""



from dataclasses import dataclass, field
from datetime import date, datetime
from typing import Optional
import numpy as np  # type: ignore
from numpy.typing import NDArray
from mugennocore.model.genre import Genre
from mugennocore.model.interfaces import IManga


@dataclass
class Manga:
    title: str
    url: str
    cover: str
    synopsis: str
    language: str = "Unknown"
    status: str = "Unknown"
    rating: float = 0.0
    last_chapter: float = 0.0
    chapters: Optional[list[str]] = None
    release_date: date = field(default_factory=datetime.now().date)
    last_update: date = field(default_factory=datetime.now().date)
    author: str = "Unknown"
    artists: str = "Unknown"
    serialization: str = "Unknown"
    genres: list[Genre] = field(default_factory=list)
    embedding: NDArray[np.float32] = field(
        default_factory=lambda: np.zeros(384, dtype=np.float32)
    )

    def update_info(
        self,
        title: Optional[str] = None,
        url: Optional[str] = None,
        synopsis: Optional[str] = None,
        cover: Optional[str] = None,
        language: Optional[str] = None,
        status: Optional[str] = None,
        rating: Optional[float] = None,
        last_chapter: Optional[float] = None,
        chapters: Optional[list[str]] = None,
        release_date: Optional[str] = None,
        last_update: Optional[str] = None,
        author: Optional[str] = None,
        artists: Optional[str] = None,
        serialization: Optional[str] = None,
        genres: Optional[list[Genre]] = None,
        embedding: Optional[NDArray[np.float32]] = None,
    ) -> None:
        """Atualiza múltiplos campos do mangá de uma vez."""
        if title is not None:
            self.title = title
        if url is not None:
            self.url = url
        if synopsis is not None:
            self.synopsis = synopsis
        if cover is not None:
            self.cover = cover
        if language is not None:
            self.language = language
        if status is not None:
            self.status = status
        if rating is not None:
            self.rating = rating
        if last_chapter is not None:
            self.last_chapter = last_chapter
        if chapters is not None:
            self.chapters = chapters
        if release_date is not None:
            self.release_date = datetime.strptime(release_date, "%Y-%m-%d").date()
        if last_update is not None:
            self.last_update = datetime.strptime(last_update, "%Y-%m-%d").date()
        if author is not None:
            self.author = author
        if artists is not None:
            self.artists = artists
        if serialization is not None:
            self.serialization = serialization
        if genres is not None:
            self.genres = genres
        if embedding is not None:
            self.embedding = embedding

    def __str__(self) -> str:
        return (
            f"{self.title}\n"
            f"{self.cover}\n"
            f"★ - {self.rating} | Caps - {self.last_chapter} | Updated on - {self.last_update}\n"
            f"Genres: {', '.join(genre.value for genre in self.genres)}"
        )

    def __repr__(self) -> str:
        return (
            f"Manga(\n"
            f"  title={repr(self.title)},\n"
            f"  url={repr(self.url)},\n"
            f"  status={repr(self.status)},\n"
            f"  rating={self.rating},\n"
            f"  last_chapter={self.last_chapter},\n"
            f"  genres={[g.value for g in self.genres]},\n"
            f"  author={repr(self.author)},\n"
            f"  release_date={self.release_date.isoformat()},\n"
            f"  embedding_shape={self.embedding.shape}\n"
            f")"
        )

    def detailed_info(self) -> str:
        """Método adicional para exibição formatada completa."""
        return f"""
📖 {self.title.upper()}
{'=' * (len(self.title) + 2)}

🔗 URL: {self.url}
✍️ Autor(es): {self.author}
🎨 Artista(s): {self.artists}
📰 Serialização: {self.serialization}
📌 Status: {self.status}
⭐ Nota: {self.rating}
📜 Último Capítulo: {self.last_chapter}
🏷️ Gêneros: {', '.join(genre.value for genre in self.genres)}
🌐 Idioma: {self.language}

📝 Sinopse:
{self.synopsis}

🗓️ Data de Lançamento: {self.release_date}
🔄 Última Atualização: {self.last_update}
"""


from mugennocore.model.interfaces import IChapter


class Chapter(IChapter):
    """Represents a Manga Chapter"""

    def __init__(
        self,
        url: str,
        download_url: str,
        title: str,
        index: float,
        release_date: str = "N/A",
    ) -> None:
        self.url = url
        self.download_url = download_url
        self.title = title
        self.index = index
        self.release_date = release_date
        self.cover = (
            None  # TODO: Implement covers to chapters based on first page of chapter
        )

    def __str__(self) -> str:
        return f"""
{self.index} - {self.title}
Doanload zip: {self.download_url}
"""

    def __repr__(self):
        return (
            f"Chapter(\n"
            f"  title={repr(self.title)},\n"
            f"  index={self.index},\n"
            f"  url={repr(self.url)},\n"
            f"  download_url={repr(self.download_url)},\n"
            f"  release_date={self.release_date},\n"
            f")"
        )



from mugennocore.model.interfaces import IPage


class Page(IPage):
    """Object to represent a Manga Page"""

    def __init__(self, img_url: str, page_index: float) -> None:
        self.img_url = img_url
        self.pade_index = page_index
        self.img_binary = None  # TODO: Implement a binary from url method

    def __str__(self) -> str:
        return f"""
{self.img_url}
Page: {self.pade_index}           
"""

    def __repr__(self):
        return (
            f"Page(\n"
            f"  img_utl={repr(self.img_url)},\n"
            f"  page_index={repr(self.pade_index)},\n"
            f"  img_binary={repr(self.img_binary)},\n"
            f")"
        )



from abc import ABC, abstractmethod
from datetime import date
from typing import Optional, Protocol, runtime_checkable
import numpy as np  # type: ignore
from numpy.typing import NDArray

from mugennocore.model.genre import Genre

@runtime_checkable
class IManga(Protocol):
    """Interface protocol to represent a Manga General Data"""
    title: str
    url: str
    cover: str
    synopsis: str
    language: str
    status: str
    rating: float
    last_chapter: float
    chapters: Optional[list[str]]
    release_date: date
    last_update: date
    author: str
    artists: str
    serialization: str
    genres: list[Genre]
    embedding: NDArray[np.float32]
    
    def update_info(self, **kwargs) -> None: ...
    def detailed_info(self) -> str: ...


class IPage(ABC):
    """Interface to represent a Manga Page"""

    def __init__(self):
        pass


class IChapter(ABC):
    """Interface to represent a Manga Chapter"""

    def __init__(self):
        pass


@runtime_checkable
class ISearchObj(Protocol):
    """Interface type-safe para objetos de busca com runtime checking."""

    title: str
    url: str
    cover_url: str
    score: float
    last_chapter: str

    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...


@runtime_checkable
class ISearchPage(Protocol):
    """Interface for search results page with pagination support."""

    # Atributos obrigatórios (type hints apenas)
    search_results: list[ISearchObj]
    pagination: list[int]

    @abstractmethod
    def __str__(self) -> str:
        """User-friendly string representation."""
        pass

    @abstractmethod
    def __repr__(self) -> str:
        """Developer-friendly string representation."""
        pass



from dataclasses import dataclass
from mugennocore.model.interfaces import ISearchPage
from mugennocore.model.search_obj import SearchObj


@dataclass
class SearchPage:
    search_results: list[SearchObj]
    pagination: list[int]

    def __str__(self) -> str:
        results_str = "\n".join(str(result) for result in self.search_results)
        return f"""
{results_str}
{'==='* len(self.pagination)}
{self.pagination}
"""

    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}(\n"
            f"  search_results={repr(self.search_results)},\n"
            f"  pagination={repr(self.pagination)}\n"
            f")"
        )
